/****************************************************************************
 * re-write by Xiaopeng Jin (xpjin@bliulab.net)
 * ACCHMM
 *****************************
 * From
 * ACCPackage 1.0
 * developed by Qiwen Dong (qwdong@fudan.edu.cn)
 * This is a standard C++ program with Standard Template Library (STL) used
 * 
 * This package converts the PSSM, generated by PSI-Blast, into 
 * a fixed-length vector by the ACC algorithm
 * ACCPSSM Reference: 
 * Qiwen Dong, Shuigeng Zhou, Jihong Guan. 
 * Protein fold recognition based on auto-cross covariance transformation.
 * Bioinformatics, 2009.
 ***************************************************************************/
#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <sstream>
#include <stdlib.h>
#include <stdio.h>
#include <cstring>
#include <cstdlib>

using namespace std;
/****************************************************************************************
 * You can switch here to decide whether to calculate the AC variable or the ACC variable
 * The package will calculate AC variable if you define AC, otherwise, the ACC variable 
 * will be calculated.
 ****************************************************************************************/
#define ACC 

bool read_profile(string fname,vector<vector<double> >& profile,string& seq);
bool read_HMMprofile(string fname,vector<vector<double> >& profile,string& seq);
//main
int main(int argc, char * argv[])
{
	int LG;
	vector<double> vec;
	int veclen,seqlen;	
	double sum,ave;
	int j,k,lg;
	string seqid,seq,fname,fnameout;
	vector<vector<double > > oneprofile;
	if(argc!=4)
	{
		cout<<"usage: programm LG PSSM_fileoutname"<<endl;
		return 0;
	}
	LG=atoi(argv[1]);
	if(LG<=0)
	{
		cout<<"invalid value of LG "<<LG<<endl;
		return 0;
	}
	fname=argv[2];

	fnameout=argv[3];


#ifdef AC
	cout<<"calculate AC variables..."<<endl;
#else
	cout<<"calculate ACC variables..."<<endl;
#endif

#ifndef AC
	vector< double > cc;
#endif

	vector<double> ave_local;
	ave_local.resize(20);

	vector<double> ac;
	ac.resize(20);


	if(!read_HMMprofile(fname,oneprofile,seq))
	{
		cout<<"cannot read profile from "<<fname<<endl;
		return 0;
	}
	seqlen=(int)oneprofile.size();
	//	cout << str(seqlen) << endl;
	//jxp
//	cout<<oneprofile.size()<<endl;
//	cout<<oneprofile[0].size()<<endl;
//	for(int i = 0; i < oneprofile.size(); i++){
//		for(int j = 0; j < oneprofile[i].size(); j++){
//			cout<<oneprofile[i][j]<< '\t';
//		}
//		cout << endl;
//	}
	//jxp End
//	cout<<">>"<<endl;
	if(LG>=seqlen)
	{
		cout<<"too large value of LG (must be less than the length of the sequence)"<<endl;
		return 0;
	}
	
#ifdef AC
	veclen=LG*20;
	vec.resize(veclen);		
#else
	int index;
	veclen=LG*400;
	vec.resize(veclen);		
	cc.resize(380);
	int k1,k2;
	double ave1,ave2;
#endif

	for(k=0;k<20;k++)
		ave_local[k]=0;
	for(j=0;j<seqlen;j++)
	{
		for(k=0;k<20;k++)
			ave_local[k]+=oneprofile[j][k];
	}
	for(k=0;k<20;k++)
		ave_local[k]/=seqlen;

	//jxp	
	//for(k=0; k<20; k++)
	//	cout<<ave_local[k]<<"\t";
	//cout<<endl;
	//cout<<ave_local.size()<<endl;
	//jxp End

	for(lg=1;lg<=LG;lg++)
	{
		//calculate AC variable
		for(k=0;k<20;k++)
		{
			ave=ave_local[k];
			sum=0;
			for(j=0;j<seqlen-lg;j++)
			{
				sum+=(oneprofile[j][k]-ave)*(oneprofile[j+lg][k]-ave);
			}
			sum/=(seqlen-lg);
			ac[k]=sum;
		}
#ifdef AC
		for(k=0;k<20;k++)
			vec[(lg-1)*20+k]=ac[k];
#else
		for(k=0;k<20;k++)
			vec[(lg-1)*400+k]=ac[k];
#endif

#ifndef AC 
		//calculate CC variable		
		for(k1=0;k1<20;k1++)
			for(k2=0;k2<20;k2++)
			{
				if(k1!=k2)
				{
					sum=0;

				ave1=ave_local[k1];
				ave2=ave_local[k2];

				sum=0;
				for(j=0;j<seqlen-lg;j++)
					sum+=(oneprofile[j][k1]-ave1)*(oneprofile[j+lg][k2]-ave2);
				sum/=(seqlen-lg);
				if(k1>k2)
					index=k1*19+k2;
				else
					index=k1*19+k2-1;					
				cc[index]=sum;
				}
			}
			for(k=0;k<380;k++)
				vec[(lg-1)*400+k+20]=cc[k];
#endif

	}
	ofstream fileout(fnameout.c_str());
	if(!fileout.is_open())
	{
		cout<<"cannot open file for out, file: "<<fnameout<<endl;
		return 0;
	}
	//cout << fname.find_last_of('/') << endl;
	int flag_n =  fname.find_last_of('/') +1;
	string file_name = fname.substr(flag_n, fname.length()-flag_n-4);
	//fileout<<fname.substr(0,fname.size()-4)<<"\t";
	fileout<<file_name<<"\t";
	for(j=0;j<veclen;j++)
	{
		fileout<<vec[j];
		if(j<(veclen-1))
			fileout<<"\t";
	}
	fileout<<"\n";
	
	cout<<"done"<<endl;
	return 1;
}
bool read_profile(string fname,vector<vector<double> >& profile,string& seq)
{
	const int aapsi2rank[20]={0,14,11,2,1,13,3,5,6,7,9,8,10,4,12,15,16,18,19,17};
	string filename,line;
	ifstream filein;
	int j,ni;
	char c;
	double tempd;
	vector<double> one_profile;
	one_profile.resize(20);

	
	filein.open(fname.c_str());
	if(!filein.is_open())
		return false;
		
	profile.clear();
	seq.clear();
	getline(filein,line);//ignore the first three lines
	getline(filein,line);
	getline(filein,line);
	while(!filein.eof())
	{
		getline(filein,line);
		if(line.empty())//terminate
			break;
		stringstream ss(line);
		ss>>ni>>c;
		seq.push_back(c);
		for(j=0;j<20;j++)
		{
			if(ss.eof())
				return false;
			ss>>tempd;
			one_profile[aapsi2rank[j]]=tempd;
		}
		profile.push_back(one_profile);
	}
	return true;
}

bool read_HMMprofile(string fname,vector<vector<double> >& profile,string& seq)
{
	//const int aapsi2rank[20]={0,14,11,2,1,13,3,5,6,7,9,8,10,4,12,15,16,18,19,17};
	const int aapsi2rank[20]={0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19};
	string filename,line;
	ifstream filein;
	int j,ni;
	char c;
	double tempd;
	vector<double> one_profile;
	one_profile.resize(20);

	
	filein.open(fname.c_str());
	if(!filein.is_open())
		return false;
		
	profile.clear();
	seq.clear();
	//getline(filein,line);//ignore the first three lines
	//getline(filein,line);
	//getline(filein,line);
	while(!filein.eof())
	{
		getline(filein,line);
		if(line.empty())//terminate
			break;
		//cout<<line<<endl;
		stringstream ss(line);
		//ss>>ni>>c;
		seq.push_back(c);
		//cout << "\n\n<<" << endl;
		//cout<<line<<endl;
		//cout << "<<" << endl;
		string abc;
	//	while(ss >> abc)
	//		cout << abc << "|";
	//	cout << endl;

		for(j=0;j<20;j++)
		{
			if(ss.eof())
				return false;
			ss>>tempd;
	//		cout << tempd << "|";
			one_profile[aapsi2rank[j]]=tempd;
		}
	//	cout << "\n___________" << endl;
	//	for(j=0;j<20;j++)
	//		cout << one_profile[j] << "\t";
	//	cout << endl;
		profile.push_back(one_profile);
	}
	return true;
}
